'*******************************************************************************
'* Programa ROBO_INTELIGENTE_AB2K.BAS
'* Desenvolvido por Mrcio Jos Soares - Arne
'* Verso 2 do Rob Inteligente
'*
'* Compilador usado: BASCOM AVR 1.11.9.8 Demo
'*
'* Obs.: Este foi desenvolvido para rodar com o Arne Basic 2k
'*******************************************************************************


'*******************************************************************************
'Configs necessrios ao microcontrolador
$crystal = 10000000
$regfile = "attiny2313.dat"
$baud = 9600

'*******************************************************************************
'Declarao das subrotinas
Declare Sub Calcula()
Declare Sub M_para()
Declare Sub M_frente()
Declare Sub M_tras()
Declare Sub M_esquerda()
Declare Sub M_direita()
Declare Sub Le_sensor()
Declare Sub R_desvia()
Declare Sub Msg_display()
Declare Sub Apaga_disp()
Declare Sub Muda_linha(byval Ln As Byte)

'*******************************************************************************
'Variveis globais
Dim A As Word                           'tamanho do pulso
Dim Desvio As Word
Dim Frente As Byte
Dim Esquerda As Byte
Dim Direita As Byte
Dim Command As Byte
Dim Clrd As Byte
Dim Lin1 As Byte
Dim Lin2 As Byte

'*******************************************************************************
'Configura portas
Config Portb = &B11111011               'PortB  sada, exceto PB2
Config Portd = &B11111110               'PortD  sada, exceto PD0

Led Alias Portb.3
Buzzer Alias Portd.2
Motor1_a Alias Portd.3
Motor1_b Alias Portd.4
Motor2_a Alias Portd.5
Motor2_b Alias Portd.6
Sensor Alias Pinb.2

'*******************************************************************************
'Configura servo e interrupo
Config Servos = 1 , Servo1 = Portb.0 , Reload = 10
Enable Interrupts

'*******************************************************************************
'Incio do programa

Frente = 100
Direita = 40
Esquerda = 250

Command = 254
Clrd = 1
Lin1 = 3
Lin2 = 2

Servo(1) = Frente

Wait 3                                  'tempo para iniciar sistemas
Printbin &HFE
Waitms 10
Printbin &H07
Waitms 10
Baud = 19200                            'seta baud a 19200 bps

Set Buzzer                              'sinal para chamar a ateno
Call Msg_display()                      'msg principal
Wait 2                                  'espera para iniciar
Call M_para()                           'pra o rob
Reset Led                               'apaga led
Reset Buzzer                            'desliga buzzer

'*******************************************************************************
'Lao principal
Do

   Call M_frente()                      'move robo sempre a frente
   Call Le_sensor()                     'le sensor de distncia

   If A <= 120 Then                      'verifica se tem alguma coisa a frente
      Call M_para()                     'pra motor
      Call Calcula()                    'calcula a distncia entre esquerda e direita
      Call R_desvia()                   'desvia do obstculo
   End If

   Toggle Led                           'pisca led enquanto anda a frente
   Waitms 100

Loop

'*******************************************************************************
'Subrotina para calcular distncias
Sub Calcula()

   Local Aux As Word

   Servo(1) = Esquerda
   Waitms 1000                          'espera servo posicionar
   Call Le_sensor()                     'l sensor
   Aux = A                              ' transfere contedo da var A

   Servo(1) = Direita
   Waitms 1000
   Call Le_sensor()                     'l sensor... A contem valor a Direita

   Call Apaga_disp()
   Print "Esquerda=" ; Str(aux);
   Call Muda_linha(2)
   Print "Direita =" ; Str(a);

   If Aux > A Then                      'decide para qual lado desviar
      Desvio = 1
   Else
      Desvio = 2
   End If


End Sub

'*******************************************************************************
'Subrotina para mover o rob a frente
Sub M_para()

   Reset Motor1_a
   Reset Motor1_b
   Reset Motor2_a
   Reset Motor2_b

End Sub

'*******************************************************************************
'Subrotina para parar o rob
Sub M_frente()

   Set Motor1_a
   Reset Motor1_b
   Set Motor2_a
   Reset Motor2_b

End Sub

'*******************************************************************************
'Subrotina para mover o rob para trs
Sub M_tras()

   Reset Motor1_a
   Set Motor1_b
   Reset Motor2_a
   Set Motor2_b

End Sub

'*******************************************************************************
'Subrotina para mover o rob para direita
Sub M_direita()

   Reset Motor1_a
   Set Motor1_b
   Set Motor2_a
   Reset Motor2_b

End Sub

'*******************************************************************************
'Subrotina para mover o rob para esquerda
Sub M_esquerda()

   Set Motor1_a
   Reset Motor1_b
   Reset Motor2_a
   Set Motor2_b

End Sub

'*******************************************************************************
'Subrotina para realizar o desvio do rob
Sub R_desvia()

   Waitms 300
   Call M_tras()                        'move para trs
   Set Buzzer                           'liga buzzer
   Wait 1                               'aguarda 1 segundo
   Call M_para()                        'pra o motor
   Waitms 100                           'espera com motor parado

   If Desvio = 1 Then                   'realiza desvio de acordo com medidas
      Call M_esquerda()                 'desvia a esquerda
   Else
      Call M_direita()                  'desvia a direita
   End If

   Waitms 300                           'espera mais 0.3 segundos
   Reset Buzzer                         'desliga buzzer

   Servo(1) = Frente
   Call Msg_display()                   'volta com msg principal

End Sub

'*******************************************************************************
'Subrotina para ler o sensor
'Obs.: esta funo faz a mesma coisa que a funo pulsein!
Sub Le_sensor()

   A = 0
   Bitwait Sensor , Set                 'espera at que pino esteja em high
   Do
      If Sensor = 1 Then                'conta tempo enquanto houver pulso
         Waitus 10                      'tempo contado em multiplos de 10 us
         A = A + 1                      'incrementa contador do tempo
      Else
         Exit Do                        'fora a sada quando pino for low
      End If
   Loop

End Sub

'*******************************************************************************
'Subrotina para apagar LCD
Sub Apaga_disp()

   Printbin &HFE                        'coloca LCD em modo comando
   Waitms 10                            'espera processar
   Printbin &H01                        'envia comando
   Waitms 10

End Sub

'*******************************************************************************
'Subrotina para muda de linha no LCD
Sub Muda_linha(byval Ln As Byte)

   Printbin &HFE                        'coloca LCD em modo comando
   Waitms 10                            'espera processar

   If Ln = 1 Then                       'se ln  1
      Printbin &H03                     'comando HOME
   Elseif Ln = 2 Then                   'mas se for 2
      Printbin &H02                     'muda para linha 2
   End If

   Waitms 10

End Sub

'*******************************************************************************
'Subrotina para imprimir mesagem no LCD
Sub Msg_display()

   Waitms 100                           'aguarda qualquer processamento anterior
   Call Apaga_disp()                    'apaga display
   Print "Robo Inteligente";            'insere mensagens
   Call Muda_linha(2)
   Print "       II      ";

End Sub


End                                     'final do programa